import React, { useState, useEffect } from 'react';
import { TableBody } from '../Table';
import { useInfiniteScroll } from './useInfiniteScroll';
import { RootFlexContainer, Wrapper, DefaultOutsideLoading } from './styles';
import { DefaultLoading } from './DefaultLoading';
import { DeafultTableLoadMore } from './DefaultLoadMore';
var LazyTableStates;
(function (LazyTableStates) {
    LazyTableStates["RENDERED"] = "rendered";
    LazyTableStates["FIRST_PAGE_LOADING"] = "firstPageLoading";
    LazyTableStates["FIRST_PAGE_LOADED"] = "firstPageLoaded";
    LazyTableStates["INFINITE_LOADING"] = "infiniteLoading";
    LazyTableStates["EMPTY"] = "empty";
})(LazyTableStates || (LazyTableStates = {}));
export var LazyTable = function (_a) {
    var hasNextPage = _a.hasNextPage, _b = _a.items, items = _b === void 0 ? [] : _b, loading = _a.loading, loadNextPage = _a.loadNextPage, isLoaded = _a.isLoaded, threshold = _a.threshold, registerScrollableList = _a.registerScrollableList, _c = _a.renderRoot, Root = _c === void 0 ? RootFlexContainer : _c, Table = _a.renderTable, Header = _a.renderHeader, renderRow = _a.renderRow, _d = _a.renderLoadingIndicator, LoadingIndicator = _d === void 0 ? DefaultLoading : _d, _e = _a.renderLoadMore, LoadMore = _e === void 0 ? DeafultTableLoadMore : _e, _f = _a.renderOutsideLoading, OutsideLoading = _f === void 0 ? DefaultOutsideLoading : _f, withPrefetchedData = _a.withPrefetchedData, _g = _a.shouldConfirmInfiniteLoading, shouldConfirmInfiniteLoading = _g === void 0 ? false : _g;
    var _h = useState(LazyTableStates.RENDERED), status = _h[0], setStatus = _h[1];
    var _j = useState(shouldConfirmInfiniteLoading), confirmLoading = _j[0], setConfirmLoading = _j[1];
    useEffect(function () {
        if (status === LazyTableStates.RENDERED && loading) {
            setStatus(LazyTableStates.FIRST_PAGE_LOADING);
        }
        else if (status === LazyTableStates.FIRST_PAGE_LOADING && !loading) {
            setStatus(LazyTableStates.FIRST_PAGE_LOADED);
        }
        else if (status === LazyTableStates.FIRST_PAGE_LOADED && loading) {
            setStatus(LazyTableStates.INFINITE_LOADING);
        }
    }, [status, loading, setStatus]);
    useEffect(function () {
        if ((status === LazyTableStates.RENDERED ||
            status === LazyTableStates.FIRST_PAGE_LOADED) &&
            !hasNextPage) {
            setStatus(LazyTableStates.EMPTY);
        }
    }, [status, hasNextPage, setStatus]);
    useEffect(function () {
        if (status === LazyTableStates.INFINITE_LOADING || status === LazyTableStates.EMPTY) {
            setConfirmLoading(false);
        }
    }, [status, setConfirmLoading]);
    var _k = useInfiniteScroll({
        loading: loading,
        hasNextPage: hasNextPage,
        itemsLength: (items === null || items === void 0 ? void 0 : items.length) || 0,
        isLoaded: isLoaded,
        onLoadMore: loadNextPage,
        registerScrollableList: registerScrollableList,
        threshold: threshold,
        withPrefetchedData: withPrefetchedData,
        confirmLoading: confirmLoading,
    }), infiniteRef = _k.ref, scrollbarWidth = _k.scrollbarWidth, showLoadMore = _k.showLoadMore, tableMaxHeight = _k.tableMaxHeight, scrollable = _k.scrollable;
    return (React.createElement(Root, null,
        React.createElement(Wrapper, { scrollbarWidth: scrollbarWidth, scrollable: scrollable },
            React.createElement(Table, null,
                React.createElement(Header, null),
                React.createElement(TableBody, { ref: infiniteRef, maxHeight: tableMaxHeight, tabIndex: 0 }, items === null || items === void 0 ? void 0 : items.map(renderRow))),
            React.createElement(OutsideLoading, null,
                showLoadMore && React.createElement(LoadMore, { loadNextPage: loadNextPage }),
                loading && React.createElement(LoadingIndicator, null)))));
};
